from ._logging import LOGGING_CONFIG
from ._mainprocess import MainProcess
from ._signalhandler import SignalHandler


__all__: list[str] = [
    'LOGGING_CONFIG',
    'MainProcess',
    'SignalHandler'
]


from typing import AsyncIterator
from typing import Protocol
from typing import TypeVar


T = TypeVar('T', bound='ITransaction')


class ITransaction(Protocol):
    __module__: str = 'tensorshield.types.protocols'

    async def transaction(
        self: T,
        transaction: T | None = None
    ) -> AsyncIterator[T]:
        ...


from ._base import BaseCanonicalException


class DoesNotExist(BaseCanonicalException):
    """This exception may be raised when attempting for perform an operation
    on a resource that does not exist.
    """
    http_status_code: int = 404


from libcanonical.types import Base64



def test_to_int():
    a = Base64.fromb64(b'AQAB')
    b = int(a)
    assert b == 65537


def test_from_int():
    a = Base64.fromint(65537)
    assert a == b'\x01\x00\x01'



import enum


class ApplicationRuntimeState(int, enum.Enum):
    BOOTING         = 0
    STARTUP         = 1
    LIVE            = 2
    READY           = 3
    BUSY            = 4
    TEARDOWN        = -1
    BOOTFAILURE     = -2
    RELOADFAILURE   = -3
    FATAL           = -99


from typing import Sequence
from typing import Generator
from typing import TypeVar

T = TypeVar('T')


def chunks(iterable: Sequence[T], n: int = 1) -> Generator[list[T], None, None]:
    l = len(iterable)
    for ndx in range(0, l, n):
        yield list(iterable[ndx:min(ndx + n, l)])


import base64

from .base64 import Base64

__all__: list[str] = [
    'Base64URLEncoded'
]



class Base64URLEncoded(Base64):
    __module__: str = 'libcanonical.types'

    @classmethod
    def b64decode(cls, value: str):
        return base64.urlsafe_b64decode(value)

    @classmethod
    def b64encode(cls, value: bytes | str) -> str:
        if isinstance(value, str):
            value = str.encode(value, 'utf-8')
        return  bytes.decode(base64.urlsafe_b64encode(value), 'ascii')